@@ -24,7 +24,8 @@ def format_date(date: datetime) -> str:
 class MetaApiWebsocketClient:
     """MetaApi websocket API client (see https://metaapi.cloud/docs/client/websocket/overview/)"""
 
-    def __init__(self, token: str, domain: str = 'agiliumtrade.agiliumtrade.ai'):
+    def __init__(self, token: str, domain: str = 'agiliumtrade.agiliumtrade.ai', request_timeout: float = 60,
+                 connect_timeout: float = 60):
         """Inits MetaApi websocket API client instance.
 
         Args:
@@ -32,6 +33,8 @@ class MetaApiWebsocketClient:
             domain: Domain to connect to, default is agiliumtrade.agiliumtrade.ai.
         """
         self._url = f'https://mt-client-api-v1.{domain}'
+        self._request_timeout = request_timeout
+        self._connect_timeout = connect_timeout
         self._token = token
         self._requestResolves = {}
         self._synchronizationListeners = {}
@@ -58,10 +61,14 @@ class MetaApiWebsocketClient:
             self._requestResolves = {}
             self._resolved = False
             result = asyncio.Future()
-
             url = f'{self._url}?auth-token={self._token}'
-            self._socket = socketio.AsyncClient(reconnection=True, reconnection_delay=1, reconnection_delay_max=5)
-            await self._socket.connect(url, socketio_path='ws')
+            self._socket = socketio.AsyncClient(reconnection=False, request_timeout=self._request_timeout)
+
+            while not self._socket.connected:
+                try:
+                    await asyncio.wait_for(self._socket.connect(url, socketio_path='ws'), timeout=self._connect_timeout)
+                except Exception:
+                    pass
 
             @self._socket.on('connect')
             async def on_connect():
@@ -69,22 +76,16 @@ class MetaApiWebsocketClient:
                 if not self._resolved:
                     self._resolved = True
                     result.set_result(None)
-                else:
-                    await self._fire_reconnected()
 
                 if not self._connected:
                     self._socket.disconnect()
 
-            @self._socket.on('reconnect')
-            async def on_reconnect():
-                await self._fire_reconnected()
-
             @self._socket.on('connect_error')
             def on_connect_error(err):
                 print(f'[{datetime.now().isoformat()}] MetaApi websocket client connection error', err)
                 if not self._resolved:
                     self._resolved = True
-                    result.set_exception(err)
+                    result.set_exception(Exception(err))
 
             @self._socket.on('connect_timeout')
             def on_connect_timeout(timeout):
@@ -109,9 +110,9 @@ class MetaApiWebsocketClient:
                     request_resolve = self._requestResolves[data['requestId']]
                     del self._requestResolves[data['requestId']]
                 else:
-                    request_resolve = asyncio.Future()
+                    request_resolve = {'accountId': data['accountId'], 'promise': asyncio.Future()}
                 self._convert_iso_time_to_date(data)
-                request_resolve.set_result(data)
+                request_resolve['promise'].set_result(data)
 
             @self._socket.on('processingError')
             def on_processing_error(data):
@@ -119,8 +120,8 @@ class MetaApiWebsocketClient:
                     request_resolve = self._requestResolves[data['requestId']]
                     del self._requestResolves[data['requestId']]
                 else:
-                    request_resolve = asyncio.Future()
-                request_resolve.set_exception(self._convert_error(data))
+                    request_resolve = {'accountId': data['accountId'], 'promise': asyncio.Future()}
+                request_resolve['promise'].set_exception(self._convert_error(data))
 
             @self._socket.on('synchronization')
             async def on_synchronization(data):
@@ -134,8 +135,7 @@ class MetaApiWebsocketClient:
         if self._connected:
             self._connected = False
             await self._socket.disconnect()
-            for request_resolve in self._requestResolves:
-                self._requestResolves[request_resolve].set_exception(Exception('MetaApi connection closed'))
+            self._drop_all_requests()
             self._requestResolves = {}
             self._synchronizationListeners = {}
 
@@ -349,7 +349,7 @@ class MetaApiWebsocketClient:
         response = await self._rpc_request(account_id, {'type': 'trade', 'trade': trade})
         return response['response']
 
-    def subscribe(self, account_id: str):
+    async def subscribe(self, account_id: str):
         """Subscribes to the Metatrader terminal events
         (see https://metaapi.cloud/docs/client/websocket/api/subscribe/).
 
@@ -359,7 +359,14 @@ class MetaApiWebsocketClient:
         Returns:
             A coroutine which resolves when subscription started.
         """
-        return self._rpc_request(account_id, {'type': 'subscribe'})
+        async def run_subscribe():
+            try:
+                await self._rpc_request(account_id, {'type': 'subscribe'})
+            except Exception as err:
+                print(f'[{datetime.now().isoformat()}] MetaApi websocket client failed to receive subscribe response',
+                      err)
+
+        asyncio.create_task(run_subscribe())
 
     def reconnect(self, account_id: str) -> Coroutine:
         """Reconnects to the Metatrader terminal (see https://metaapi.cloud/docs/client/websocket/api/reconnect/).
@@ -487,9 +494,17 @@ class MetaApiWebsocketClient:
         self._reconnectListeners = []
 
     async def _reconnect(self):
-        while (not self._socket.connected) and self._connected:
-            await asyncio.sleep(1)
-            await self._socket.connect()
+        reconnected = False
+        while self._connected and not reconnected:
+            try:
+                await self._socket.disconnect()
+                url = f'{self._url}?auth-token={self._token}'
+                await asyncio.wait_for(self._socket.connect(url, socketio_path='ws'), timeout=self._connect_timeout)
+                reconnected = True
+                await self._fire_reconnected()
+                await self._socket.wait()
+            except Exception:
+                pass
 
     async def _rpc_request(self, account_id: str, request: dict) -> Coroutine:
         if not self._connected:
@@ -497,11 +512,13 @@ class MetaApiWebsocketClient:
 
         request_id = ''.join(random.choice(string.ascii_lowercase) for i in range(32))
 
-        self._requestResolves[request_id] = asyncio.Future()
+        self._requestResolves[request_id] = {'accountId': account_id, 'promise': asyncio.Future()}
         request['accountId'] = account_id
         request['requestId'] = request_id
         await self._socket.emit('request', request)
-        return await self._requestResolves[request_id]
+        resolve = await asyncio.wait_for(self._requestResolves[request_id]['promise'],
+                                         timeout=self._request_timeout)
+        return resolve
 
     def _convert_error(self, data) -> Exception:
         if data['error'] == 'ValidationError':
@@ -540,6 +557,7 @@ class MetaApiWebsocketClient:
                         except Exception as err:
                             print('Failed to notify listener about connected event', err)
             elif data['type'] == 'disconnected':
+                self._drop_all_requests(data['accountId'])
                 if data['accountId'] in self._synchronizationListeners:
                     for listener in self._synchronizationListeners[data['accountId']]:
                         try:
@@ -686,6 +704,15 @@ class MetaApiWebsocketClient:
         except Exception as err:
             print('Failed to process incoming synchronization packet', err)
 
+    def _drop_all_requests(self, account_id: str = None):
+        for request_resolve in self._requestResolves:
+            if (not self._requestResolves[request_resolve]['promise'].done()) and \
+                    (self._requestResolves[request_resolve]['accountId'] == account_id if account_id else True):
+                exception_message = f'Account {account_id} has disconnected from MetaApi, thus all requests to ' \
+                                    f'this account were cancelled' if account_id else 'MetaApi connection closed'
+                self._requestResolves[request_resolve]['promise'].set_exception(
+                    NotConnectedException(exception_message))
+
     async def _fire_reconnected(self):
         for listener in self._reconnectListeners:
             try:
