@@ -3,6 +3,7 @@ from socketio import AsyncServer
 from aiohttp import web
 from ..models import date
 import pytest
+import asyncio
 import copy
 from urllib.parse import parse_qs
 from mock import MagicMock, AsyncMock
@@ -40,6 +41,7 @@ class FakeServer:
 
 @pytest.fixture(autouse=True)
 async def run_around_tests():
+    FinalMock.__await__ = lambda x: async_magic_close().__await__()
     fake_server = FakeServer()
     await fake_server.start()
     global client
@@ -455,7 +457,6 @@ class TestMetaApiWebsocketClient:
     @pytest.mark.asyncio
     async def test_connect_to_terminal(self):
         """Should connect to MetaTrader terminal."""
-
         request_received = False
 
         @sio.on('request')
@@ -463,10 +464,9 @@ class TestMetaApiWebsocketClient:
             if data['type'] == 'subscribe' and data['accountId'] == 'accountId':
                 nonlocal request_received
                 request_received = True
-                await sio.emit('response', {'type': 'response', 'accountId': data['accountId'],
-                                            'requestId': data['requestId']})
 
         await client.subscribe('accountId')
+        await asyncio.sleep(0.05)
         assert request_received
 
     @pytest.mark.asyncio
@@ -637,13 +637,25 @@ class TestMetaApiWebsocketClient:
     @pytest.mark.asyncio
     async def test_process_disconnected_synchronization_event(self):
         """Should process disconnected synchronization event."""
+
+        async def test_close():
+            assert client._requestResolves['test']['promise'].exception().args[0] == \
+                   'Account accountId has disconnected from MetaApi, thus all requests to this account were cancelled'
+            assert not client._requestResolves['test2']['promise'].done()
+            await client.close()
+
+        FinalMock.__await__ = lambda x: test_close().__await__()
+
         listener = MagicMock()
         listener.on_disconnected = FinalMock()
 
+        client._requestResolves['test'] = {'accountId': 'accountId', 'promise': asyncio.Future()}
+        client._requestResolves['test2'] = {'accountId': 'accountId2', 'promise': asyncio.Future()}
         client.add_synchronization_listener('accountId', listener)
         await sio.emit('synchronization', {'type': 'disconnected', 'accountId': 'accountId'})
         await client._socket.wait()
         listener.on_disconnected.assert_called_with()
+        assert 'test' not in client._requestResolves
 
     @pytest.mark.asyncio
     async def test_synchronize_with_metatrader_terminal(self):
@@ -906,6 +918,27 @@ class TestMetaApiWebsocketClient:
         listener.on_history_order_added.assert_called_with(update['historyOrders'][0])
         listener.on_deal_added.assert_called_with(update['deals'][0])
 
+    @pytest.mark.asyncio
+    async def test_timeout_on_no_response(self):
+        """Should return timeout error if no server response received."""
+
+        trade = {
+            'actionType': 'ORDER_TYPE_SELL',
+            'symbol': 'AUDNZD',
+            'volume': 0.07
+        }
+
+        @sio.on('request')
+        async def on_request(sid, data):
+            pass
+
+        try:
+            await client.trade('accountId', trade)
+            Exception('TimeoutError expected')
+        except Exception as err:
+            assert err.__class__.__name__ == 'TimeoutError'
+            await client.close()
+
     @pytest.mark.asyncio
     async def test_subscribe_to_market_data_with_mt_terminal(self):
         """Should subscribe to market data with MetaTrader terminal."""
